	;**********************************************************************
	;                                                                     *
	;**********************************************************************
	;    Filename:	    forno.asm                                           *
	;    Date:                                                            *
	;    File Version:                                                    *
	;                                                                     *
	;    Author:                                                          *
	;    Company:                                                         *
	;                                                                     *
	;**********************************************************************
	;    Files Required: P16F677.INC                                      *
	;                                                                     *
	;**********************************************************************
	;    PIC 16F677 pinout:                                               *
	;                          |--\/--|                                   *
	;                     Vdd -|1   20|- Vss                              *
	;    RA5/T1CKI/OSC1/CLKIN -|2   19|- RA0/AN0/C1IN+/ICSPDAT/ULPWU      *
	; RA4/AN3/T1G/OSC2/CLKOUT -|3   18|- RA1/AN1/C12IN0-/VREF/ICSPCLK     *
	;            RA3/MCLR/VPP -|4   17|- RA2/AN2/T0CKI/INT/C1OUT          *
	;                     RC5 -|5   16|- RC0/AN4/C2IN+                    *
	;               RC4/C2OUT -|6   15|- RC1/AN5/C12IN1-                  *
	;           RC3/AN7C12IN3 -|7   14|- RC2/AN6/C12IN2-                  *
	;              RC6/AN8/SS -|8   13|- RB4/AN10/SDI/SDA                 *
	;             RC7/AN9/SDO -|9   12|- RB5/AN11                         *
	;                     RB7 -|10  11|- RB6/SCK/SCL                      *
	;                          --------                                   *
	;                                                                     *
	;**********************************************************************

;/****************************************************************************
;* DESCRIPTION: System definitions.
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

	ERRORLEVEL -302 ;remove message about using proper bank

	LIST  P=16F677
	INCLUDE <P16F677.INC>
	__CONFIG _BOR_OFF & _FCMEN_OFF & _IESO_OFF & _CPD_OFF & _CP_OFF & _MCLRE_OFF & _PWRTE_ON & _WDT_OFF & _INTOSCIO

;/****************************************************************************
;* DESCRIPTION: Macros used in program.
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

SALVA_CONTEXTO  MACRO
	MOVWF	W_TEMP			; save W content to W_TEMP
	SWAPF	STATUS,W		; put STATUS in W...
	MOVWF	STATUS_TEMP		; ... Then save it in STATUS_TEMP
	ENDM
RESTAURA_CONTEXTO  MACRO
	SWAPF	STATUS_TEMP,W	; Restaure STATUS content...
	MOVWF	STATUS			; ... from STATUS_TEMP
	SWAPF	W_TEMP,F		; Restaure W content...
	SWAPF	W_TEMP,W		; ... from W_TEMP.
	ENDM

;/****************************************************************************
;* DESCRIPTION: Program variables and constant definitions.
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

TEMPER		EQU 0x20						; Store temperature
AUX			EQU	0x21						; REGISTRADOR AUXILIAR DE USO GERAL
D0			EQU 0x23						; Temporary for display
D1			EQU 0x24						; Temporary for display
TEMP		EQU 0x25						; General use variable
TEMP1		EQU 0x26						; General use variable
FLAGS		EQU 0x27						; General boolean flags (8 flags)
FLAGSOP		EQU 0x28						; General boolean flags (8 flags)
W_TEMP		EQU 0x29						; Temporary W register
STATUS_TEMP	EQU 0x2A						; Temporary STATUS register
T1_CONTA	EQU 0x2B						; Aux. Timer1 counter
DCNT0		EQU	0x2C						; Temp counter for buzzer alarm
DCNT1		EQU	0x2D						; Temp counter for buzzer alarm
TEMPK1		EQU 0x2E						; For delay routine
TEMPK2		EQU 0x2F						; For delay routine
UNITS		EQU	0x30						; For keys routines
TENS		EQU	0x31						; For keys routines
SW_FLAG		EQU	0x32						; For keys routines
FASTCOUNT	EQU	0x33						; For keys routines
TMR_1MS		EQU	0x34						; Counter for 1 ms period
TMR_100MS	EQU	0x35						; Counter for 100 ms period
DISPTMP		EQU	0x36						; Temporary display's 
BEEPCNT		EQU	0x37						; Number of beeps
TMR_DISP	EQU	0x38						; Timer int Display
TMR_LEDS	EQU	0x39						; Timer int leds
TEMPO0		EQU	0x3A
TEMPO1		EQU	0x3B
UNIDADE		EQU 0x3C						; ARMAZENA VALOR DA UNIDADE
DEZENA		EQU	0x3D						; ARMAZENA VALOR DA DEZENA
CENTENA		EQU 0x3E						; ARMAZENA VALOR DA CENTENA
STATE		EQU 0x3F						; State machine's register
INT_STATUS	EQU	0x40						; State Machine's display enable.

#DEFINE		BUZZERON			FLAGS,0		; Buzzer on?
#DEFINE		LEDONOFF			FLAGS,1		; Led onoff active?
#DEFINE		TOASTON				FLAGS,2		; Toaster ON?
#DEFINE		LIGHTON				FLAGS,3		; Light ON?

#DEFINE		DISPTMP7			DISPTMP,7
#DEFINE		COOKING				FLAGSOP,0	; Are we Cooking?
#DEFINE		TEMPMODE			FLAGSOP,1	; Are we in temperature set mode?
#DEFINE		TIMEMODE			FLAGSOP,2	; Are we in timer set mode?
#DEFINE		HOTTING				FLAGSOP,3	; In Pre-Hotting mode?
#DEFINE		ERRORMOD			FLAGSOP,4	; In Error Mode?
#DEFINE		TIMEFORDISPLAY		FLAGSOP,5	; Time to show display?
#DEFINE		TIMEFORLEDS			FLAGSOP,6	; Time to show leds?

#DEFINE		DISP3INT			INT_STATUS,0; Time to show Disp3
#DEFINE		DISP2INT			INT_STATUS,1; Time to show Disp2
#DEFINE		DISP1INT			INT_STATUS,2; Time to show Disp1
#DEFINE		LEDENINT			INT_STATUS,3; Time to shw Leds

; Keys
#DEFINE		KEY_UP				PORTB,7		; KEY_UP
#DEFINE		KEY_TIMER			PORTB,6		; KEY_TIMER
#DEFINE		KEY_ONOFF			PORTB,5		; KEY_ONOFF
#DEFINE		KEY_TEMP			PORTA,4		; KEY_TEMP
#DEFINE		KEY_DWN				PORTA,2		; KEY_DWN
#DEFINE		KEY_LIGHT			PORTB,4		; KEY_LIGHT
#DEFINE		KEY_TOASTER			PORTA,5		; KEY_TOASTER

; Leds
#DEFINE		LED_CMN				PORTC,2		; Leds Anodes

#DEFINE		LED_UP				PORTA,4		; Led UP
#DEFINE		LED_TIMER			PORTB,6		; Led TIMER
#DEFINE		LED_TEMP			PORTB,5		; Led TEMP
#DEFINE		LED_DWN				PORTA,2		; Led DWN
#DEFINE		LED_LIGHT			PORTB,4		; Led LIGHT
#DEFINE		LED_TOASTER			PORTA,5		; Led TOASTER
#DEFINE		LED_ONOFF			PORTC,6		; Led ONOFF

; Displays
#DEFINE		DISP1				PORTC,3		; Anode display DIS0
#DEFINE		DISP2				PORTC,4		; Anode display DIS1
#DEFINE		DISP3				PORTC,5		; Anode display DIS2
#DEFINE		SEG_A				PORTA,2		; Display semgment a
#DEFINE		SEG_B				PORTA,5		; Display semgment b
#DEFINE		SEG_C				PORTB,7		; Display semgment c
#DEFINE		SEG_D				PORTB,5		; Display semgment d
#DEFINE		SEG_E				PORTA,4		; Display semgment e
#DEFINE		SEG_F				PORTB,6		; Display semgment f
#DEFINE		SEG_G				PORTB,4		; Display semgment g

; Reles
#define		RELE_MAIN			PORTC,0		; Main heater
#define		RELE_TOAST			PORTA,1		; Toaster
#define		RELE_LIGHT			PORTA,0		; Light

; Buzzer
#DEFINE		BUZZER				PORTC,0		; Buzzer

; Temperature Sensor
#DEFINE		TEMPSENS			PORTC,1		; Temperature Sensor

;CPU Frequency Reference
#define		MHZ					D'4'		; processor frequency in MHz

;/****************************************************************************
;* DESCRIPTION: Got Start of program.
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;***************************************************************************** 
	ORG		0x0000
	GOTO	START

;****************************************************************************
;* DESCRIPTION: Interrupts service (Timer1, Timer2,...).
;*				For Timer 1 Interrupt:
;* Overflow: 1/((FHz/4)/T1_Prescaler) = 1/(4000000/1) = 1/4000000 = 0,000001s = 1us.
;* For an overflow of 1ms:
;* 0,001 = 0,000001 * X
;* X = 0,001/0,000001 => X = 1000
;* For an overflow of 1ms, preload Timer1 with: 65536-1000, wich is: 64536
;* 64536 = (0xFC18) -> 0xFC in TMR1H and 0x18 in TMR1L.
;* Thus:
;* f(interrupt)=1/(freq_cpu/4/prescaler/preloadvalue)
;* As I want one interrupt at every 1 ms:
;* f(interrupt)=1/(4000000/4/1/1000)
;* f(interrupt)= 1/1000 times/second => T(f) = 1/f = 0,001 (1 ms).
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       interrupt every 1ms
;*****************************************************************************
	ORG 0x0004
	SALVA_CONTEXTO			; Save Context
	BCF		STATUS,RP0		; Bank 0
	BCF		STATUS,RP1		;
	BCF		PIR1,TMR1IF		; Clear Timer1 Interrupt flag

	MOVLW	0x18			;
	MOVWF	TMR1L			;
	MOVLW	0xFC			;
	MOVWF	TMR1H			; Init TIMER1 with decimal 64536 (interrupt every 1ms)

	INCF	STATE,F			; Icrement State Machine's register
	MOVLW	D'2'
	XORWF	STATE,W			; Time to enable Display3?
	BTFSC	STATUS,Z		;
	GOTO	ENABLE_DSP3		; Yes, do it!
	MOVLW	D'4'			;
	XORWF	STATE,W			; Time to enable Display2?
	BTFSC	STATUS,Z		;
	GOTO	ENABLE_DSP2		; Yes, do it!
	MOVLW	D'6'			;
	XORWF	STATE,W			; Time to enable Display1?
	BTFSC	STATUS,Z		;
	GOTO	ENABLE_DSP1		; Yes, do it!
	MOVLW	D'8'			;
	XORWF	STATE,W			; Time to enable Leds?
	BTFSC	STATUS,Z		;
	GOTO	ENABLE_LED		; Yes, do it!

	GOTO	CONTINUEINT		; Just in case. Should never get here!

ENABLE_DSP3
	BCF		LEDENINT		; Disable LEDS
	BSF		DISP3INT		; Enable Disp3
	GOTO	CONTINUEINT		; Exit interrupt
ENABLE_DSP2
	BCF		DISP3INT		; Disable DISP3
	BSF		DISP2INT		; Enable DISP2
	GOTO	CONTINUEINT		; Exit interrupt
ENABLE_DSP1
	BCF		DISP2INT		; Disable DISP2
	BSF		DISP1INT		; Enable DISP1
	GOTO	CONTINUEINT		; Exit interrupt
ENABLE_LED
	BCF		DISP1INT		; Disable DISP1
	BSF		LEDENINT		; Enable LEDS
	CLRF	STATE

CONTINUEINT:
	INCF	TMR_1MS,F		; Increment TR_1MS (1 ms counter)
	MOVLW	D'100'
	XORWF	TMR_1MS,W		; Is TMR_1MS = 100?
	BTFSC	STATUS,Z		; No, skip if TMR_1MS < 100
	GOTO	INC100MS		; Yes, zero TMR_1MS and inc TMR_100MS
	GOTO	END_INT			; No, exit interrupt

INC100MS
	MOVLW	D'0'			;
	MOVWF	TMR_1MS			; Zero TMR_1MS
	INCF	TMR_100MS,F		; Increment TMR_100MS (100 ms counter)
	MOVLW	D'5'			; 500 ms?
	XORWF	TMR_100MS,W		; Is TMR_100MS = 5 (500ms)?
	BTFSC	STATUS,Z		; No, skip if TMR_100MS < 5
	GOTO	INCCONTA		; Yes, zero TMR_100MS and inc 0,5 seconds counter
	GOTO	END_INT			; No, Exit interrupt

INCCONTA:
	MOVLW	D'0'			;
	MOVWF	TMR_100MS		; Zero 100 ms counter (reached 500ms)	
	BSF		STATUS,RP0		; Bank 1
	BCF		STATUS,RP1		;
	BCF		LED_ONOFF		; LED_ONOFF output

	BCF		STATUS,RP0		; Bank 0
	BCF		STATUS,RP1		;

	MOVLW	D'1'			; Verify if ...
	XORWF	T1_CONTA,W		; passed 1/2 second ...
	BTFSC	STATUS,Z		; 
	GOTO	LEDON			; Led ONOFF is ON? Goto LEDON
LEDOFF:
	BCF		LED_ONOFF		; Turn OFF LEDON_OFF
	GOTO	LEDOUT			; ok, done...
LEDON:
	BSF		LED_ONOFF		; Turn ON LED_ONOFF
	BSF		BUZZERON		; Enable BEEP
LEDOUT:
	DECFSZ	T1_CONTA,F		; Decrement the Counter rRegister
	GOTO	END_INT			; If diferent of zero, exit interrupt
	MOVLW	2				; 
	MOVWF	T1_CONTA		; Restart Counter Register with 2	
END_INT:
	RESTAURA_CONTEXTO		; Restore Context
	RETFIE					; Return from interrupt

;/****************************************************************************
;* DESCRIPTION: Decode to decimal, from an 8 bits number
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

BIN8DEC:
	MOVWF	D0				; copia o número a ser convertido para a variável de unidades
	CLRF	D1				; limpa as dezenas
BIN8DEC_2:
	MOVLW	D'10'			; subtrai 10 decimal ...
	SUBWF	D0,W			; ... do valor a ser convertido e guarda o resultado em W
	BTFSS	STATUS,C		; o resultado é positivo ou zero ?
	RETURN					; não ? então retorna, a conversão terminou
	MOVWF	D0				; sim ? então copia o resultado em W para as unidades ...
	INCF	D1,F			; incrementa um nas dezenas 
	GOTO	BIN8DEC_2		; e reinicia o ciclo

;/****************************************************************************
;* DESCRIPTION: binary_to_bcd - 8-bits
;* INPUT:		TEMPER  - 8-bit binary number
;* RETURN:      CENTENA - the hundreds digit of the BCD conversion
;*				DEZENA  - the tens digits of the BCD conversion
;*				UNIDADE - the ones digits of the BCD conversion
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 
BINARY_TO_BCD:

	MOVWF	AUX				; SALVA VALOR A CONVERTER EM AUX
	CLRF	UNIDADE
	CLRF	DEZENA
	CLRF	CENTENA			; RESETA REGISTRADORES
	MOVF	AUX,F
	BTFSC	STATUS,Z		; VALOR A CONVERTER = 0 ?
	RETURN					; SIM - RETORNA
							; NÃO
	INCF	UNIDADE,F		; INCREMENTA UNIDADE
	MOVF	UNIDADE,W
	XORLW	0X0A
	BTFSS	STATUS,Z		; UNIDADE = 10d ?
	GOTO	$+3				; NÃO
							; SIM
	CLRF	UNIDADE			; RESETA UNIDADE
	INCF	DEZENA,F		; INCREMENTA DEZENA
	MOVF	DEZENA,W
	XORLW	0X0A
	BTFSS	STATUS,Z		; DEZENA = 10d ?
	GOTO	$+3				; NÃO
							; SIM
	CLRF	DEZENA			; RESETA DEZENA
	INCF	CENTENA,F		; INCREMENTA CENTENA
	DECFSZ	AUX,F			; FIM DA CONVERSÃO ?
	GOTO	$-.14			; NÃO - VOLTA P/ CONTINUAR CONVERSÃO
	RETURN					; SIM

;/****************************************************************************
;* DESCRIPTION: Set led onoff within segments decode
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

DECOD_SEGMENTS
	MOVWF	DISPTMP			; Store W in DISPTMP
TEST_A:
	BTFSS	DISPTMP,0
	GOTO	SEG_A_ON
	GOTO	SEG_A_OFF
SEG_A_ON
	BCF		SEG_A
	GOTO	TEST_B
SEG_A_OFF:
	BSF		SEG_A
TEST_B:
	BTFSS	DISPTMP,1
	GOTO	SEG_B_ON
	GOTO	SEG_B_OFF
SEG_B_ON
	BCF		SEG_B
	GOTO	TEST_C
SEG_B_OFF:
	BSF		SEG_B
TEST_C:
	BTFSS	DISPTMP,2
	GOTO	SEG_C_ON
	GOTO	SEG_C_OFF
SEG_C_ON
	BCF		SEG_C
	GOTO	TEST_D
SEG_C_OFF:
	BSF		SEG_C
TEST_D:
	BTFSS	DISPTMP,3
	GOTO	SEG_D_ON
	GOTO	SEG_D_OFF
SEG_D_ON
	BCF		SEG_D
	GOTO	TEST_E
SEG_D_OFF:
	BSF		SEG_D
TEST_E:
	BTFSS	DISPTMP,4
	GOTO	SEG_E_ON
	GOTO	SEG_E_OFF
SEG_E_ON
	BCF		SEG_E
	GOTO	TEST_F
SEG_E_OFF:
	BSF		SEG_E
TEST_F:
	BTFSS	DISPTMP,5
	GOTO	SEG_F_ON
	GOTO	SEG_F_OFF
SEG_F_ON
	BCF		SEG_F
	GOTO	TEST_G
SEG_F_OFF:
	BSF		SEG_F
TEST_G:
	BTFSS	DISPTMP,6
	GOTO	SEG_G_ON
	GOTO	SEG_G_OFF
SEG_G_ON
	BCF		SEG_G
	GOTO	END_SEGS
SEG_G_OFF:
	BSF		SEG_G
END_SEGS:
	RETURN

;/****************************************************************************
;* DESCRIPTION: Decodes value in W register to display segments.
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

DECOD_DISPLAY:
	MOVWF	TEMP			; armazena o número na variável deslocamento
	MOVLW	LOW TABELA		; copia em W os 8 bits LSB do endereço da TABELA
	ADDWF	TEMP,F			; adiciona o valor à variável DESLOCAMENTO
	MOVLW	HIGH TABELA		; copia em W os 5 bits superiores do endereço da TABELA
	BTFSC	STATUS,C		; testa para ver se a soma anterior transbordou 
	ADDLW	0x01			; se transbordou, soma 1 ao W
	MOVWF	PCLATH			; acerta o PCLATH de acordo com o endereço da TABELA
	MOVF	TEMP,W			; copia o valor do DESLOCAMENTO para o W
	MOVWF	PCL				; copia o W para o PCL (desvia para a tabela)

TABELA:
;       A
;	 _______
;   |       |
; F |       | B
;   |   G   |
;    -------
;   |       |
; E |       | C
;   |_______|
;       D

	RETLW	B'11000000'		; 0    |-|-|F|E|D|C|B|A|-      0
	RETLW	B'11111001'		; 1    |-|-|-|-|-|C|B|-|-      1
	RETLW	B'10100100'		; 2    |-|G|-|E|D|-|B|A|-      2
	RETLW	B'10110000'		; 3    |-|G|-|-|D|C|B|A|-      3
	RETLW	B'10011001'		; 4    |-|G|F|-|-|C|B|-|-      4
	RETLW	B'10010010'		; 5    |-|G|F|-|D|C|-|A|-      5
	RETLW	B'10000010'		; 6    |-|G|F|E|D|C|-|A|-      6
	RETLW	B'11111000'		; 7    |-|-|-|-|-|C|B|A|-      7
	RETLW	B'10000000'		; 8    |-|G|F|E|D|C|B|A|-      8
	RETLW	B'10011000'		; 9    |-|G|F|-|-|C|B|A|-      9
	RETLW	B'11110111'		; A    |-|G|F|E|-|C|B|A|-     10
	RETLW	B'10000000'		; B    |-|G|F|E|D|C|-|-|-     11
	RETLW	B'11000110'		; C    |-|-|F|E|D|-|-|A|-     12
	RETLW	B'10100001'		; D    |-|G|-|E|D|C|B|-|-     13
	RETLW	B'10000110'		; E    |-|G|F|E|D|-|-|A|-     14
	RETLW	B'10001110'		; F    |-|G|F|E|-|-|-|A|-     15

;/****************************************************************************
;* DESCRIPTION: Little Delay to let display show digit.
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

DELAY_MS:					; Rotina de atraso - colocar em W o número de ms.
	MOVWF	TEMPO1			; W = número de ms de atraso.
	MOVLW	D'250'
	MOVWF	TEMPO0
	DECFSZ	TEMPO0,F
	GOTO	$-1
	DECFSZ	TEMPO1,F
	GOTO	$-5
	RETURN

;/****************************************************************************
;* DESCRIPTION: Check if any key is active (pressed)
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

CHECKKEYS:
	BSF		STATUS,RP0		; Bank 1
	BCF		STATUS,RP1		;

	MOVLW	B'00111100'		;
	MOVWF	TRISA			;

	MOVLW	B'11110000'		;
	MOVWF	TRISB			;

	MOVLW	B'00110100'
	MOVWF	WPUA

	BCF		STATUS,RP0		; Bank 2
	BSF		STATUS,RP1		;
	MOVLW	B'11110000'
	MOVWF	WPUB

	BSF		STATUS,RP0		; Bank 1
	BCF		STATUS,RP1		;

	BSF		OPTION_REG,7	; Set RABPU flag

	BCF		STATUS,RP0		; Bank 0
	BCF		STATUS,RP1		;

	BTFSS	KEY_UP			; KEY_UP pressed?
	CALL	FASTUP			; Yes
	BTFSS	KEY_TIMER		; KEY_TIMER pressed?
	CALL	TIMERSET		; Yes
	BTFSS	KEY_ONOFF		; KEY_ONOFF pressed?
	CALL	ONOFF			; Yes
	BTFSS	KEY_TEMP		; KEY_TEMP pressed?
	CALL	TEMPSET			; Yes
	BTFSS	KEY_DWN			; KEY_DWN pressed?
	CALL	FASTDWN			; Yes
	BTFSS	KEY_LIGHT		; KEY_LIGHT pressed?
	CALL	LIGHTSET		; Yes
	BTFSS	KEY_TOASTER		; KEY_TOASTER pressed?
	CALL	TOASTERSET		; Yes


	BSF		STATUS,RP0		; Bank 1
	BCF		STATUS,RP1		;

	MOVLW	B'00000000'
	MOVWF	WPUA

	BCF		STATUS,RP0		; Bank 2
	BSF		STATUS,RP1		;
	MOVLW	B'00000000'
	MOVWF	WPUB

	BSF		STATUS,RP0		; Bank 1
	BCF		STATUS,RP1		;

	BCF		OPTION_REG,7	; Set RABPU flag

	MOVLW	B'00001000'		;
	MOVWF	TRISA			;

	MOVLW	B'00000000'		;
	MOVWF	TRISB			;

	BCF		STATUS,RP0		; Bank 0
	BCF		STATUS,RP1		;

	RETURN

;/****************************************************************************
;* DESCRIPTION: Process Key Up
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

FASTUP:						;

	BTFSS	SW_FLAG,2		; First time through loop?
	GOTO	FU_2			; Yes
	BTFSC	SW_FLAG,7		; Has 5Hz bit been set?
	GOTO	FU_3			;
FU_1:						;
	INCFSZ	FASTCOUNT,f		; Increment FASTCOUNT
	MOVLW	d'100'			;
	XORWF	FASTCOUNT,w		;
	BTFSS	STATUS,2		; Reached 100 loops?
	RETLW	00				;
	CLRF	FASTCOUNT		;
	BSF		SW_FLAG,7		; Set bit for 5Hz incrementing	

FU_2:						;
	BSF		SW_FLAG,2		; Up button has been pressed
	INCF	TEMPER,F		; incrementa temperatura
	MOVLW	D'251'			; verifica ...
	XORWF	TEMPER,W		; se passou de 250 graus ...
	BTFSS	STATUS,Z		;
	GOTO	FIMTECLAUP		; se não, sai da rotina
	CLRF	TEMPER			; se passsou ...

FIMTECLAUP:

	RETLW	00				; Display passes 99 but not below 0

FU_3:						;
	INCFSZ	FASTCOUNT,f		; Increment FASTCOUNT
	MOVLW	d'20'			; era 5
	XORWF	FASTCOUNT,w		;
	BTFSS	STATUS,2		; Reached 5 loops?
	RETLW	00				;
	CLRF	FASTCOUNT		;
	GOTO	FU_2			;

;/****************************************************************************
;* DESCRIPTION: Process Key Down
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

FASTDWN:					;
	DECF	TEMPER,F		; decrementa temperatura em 1 grau
	RETURN					; RETURN
TIMERSET:
	RETURN					; RETURN
ONOFF:
	RETURN					; RETURN
TEMPSET:
	RETURN					; RETURN
LIGHTSET:
	RETURN					; RETURN
TOASTERSET:
	RETURN					; RETURN

;/****************************************************************************
;* DESCRIPTION: Program Initialization
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

START:
	BCF		STATUS,RP0		; Bank 2
	BSF		STATUS,RP1		;
	CLRF	ANSEL			; digital I/O
	CLRF	ANSELH			;	"	"	"
	BCF		STATUS,RP0		; Bank 2
	BSF		STATUS,RP1		;
	MOVLW	B'00000000'
	MOVWF	CM1CON0			; disable comparator
	MOVLW	B'00000000'		; Not necessary?
	MOVWF	CM2CON0			; disable 2nd comparator
	MOVLW	B'00000000'		; Not necessary?
	MOVWF	CM2CON1			; disable 3rd comparator
	BSF		STATUS,RP0		; Bank 1
	BCF		STATUS,RP1		;
	MOVLW	B'00001000'		;
	MOVWF	TRISA			;
	MOVLW	B'00000000'		;
	MOVWF	TRISB			;
	MOVLW	B'10000010'		;
	MOVWF	TRISC			;
	BCF		STATUS,RP0		; Bank 0
	BCF		STATUS,RP1		;
	BCF		SSPCON,SSPEN	; Disable Synchronous Serial Port

	; Desliga todos os reles, displays e leds
	BSF		LED_ONOFF		; Led ONOFF OFF
	BSF		RELE_MAIN		; Main heater OFF
	BSF		RELE_TOAST		; Toaster OFF
	BSF		RELE_LIGHT		; Light OFF
	BSF		LED_CMN			; Leds off
	BSF		DISP1			; Display1 off
	BSF		DISP2			; Display2 off
	BSF		DISP3			; Display3 off
	BSF		BUZZER			; Buzzer off
	MOVLW	D'180'			; Init temperature with 180 degrees Celsius
	MOVWF	TEMPER
	CLRF	STATE			; Init State Machine's register
	CLRF	FLAGS			; ...
	MOVLW	2				;
	MOVWF	T1_CONTA		;
	; Init Timer1
	MOVLW	B'00000000'		; Configure TIMER1 for internal clock and prescaler 1:1
	MOVWF	T1CON			; Init Timer1
	MOVLW	0x18			;
	MOVWF	TMR1L			;
	MOVLW	0xFC			;
	MOVWF	TMR1H			; Init TIMER1 with decimal 64536 (interrupt every 1ms)
	BSF		STATUS,RP0		; Bank 1
	BCF		STATUS,RP1		;
	MOVLW   B'00000001'		; Enable TIMER1 Overflow Interrupt
	MOVWF	PIE1
	BCF		STATUS,RP0		; Bank 0
	BCF		STATUS,RP1		;
	CLRF	PIR1			; Clear Peripheral Interrupt Flags
	BSF		INTCON, PEIE	; Enable Peripheral Interrupts
	BSF		T1CON, TMR1ON	; Start TMR1
	BSF		INTCON, GIE		; Enable Global Interrupts


;/****************************************************************************
;* DESCRIPTION: Main Program
;* RETURN:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

MAIN_LOOP:
	MOVF	TEMPER,W		; Put temperature in W
	CALL	BINARY_TO_BCD	; Decode Temperature to bcd

TESTDISP3:
	BTFSS	DISP3INT		; Show display 3 (Units)?
	GOTO	TESTDISP2
	BSF		LED_CMN			; Leds off
	MOVF	UNIDADE,W		; copia o dígito em W
	CALL	BIN8DEC			;
	MOVF	D0,W			;
	CALL	DECOD_DISPLAY	; decodifica o dígito
	CALL	DECOD_SEGMENTS
	BCF		DISP1			; ativa display 1
	CLRW					; W=0
	GOTO	MAINCONTINUE

TESTDISP2:
	BTFSS	DISP2INT		; Show display 2 (Tens)?
	GOTO	TESTDISP1
	BSF		DISP1			; Display1 off
	MOVF	DEZENA,W		; copia o dígito em W
	CALL	BIN8DEC			;
	MOVF	D0,W			;
	CALL	DECOD_DISPLAY	; decodifica o dígito
	CALL	DECOD_SEGMENTS
	BCF		DISP2			; ativa display 2
	CLRW					; W=0
	GOTO	MAINCONTINUE

TESTDISP1:
	BTFSS	DISP1INT		; Show dispay 1 (Hundreds)?
	GOTO	TESTLEDS
	BSF		DISP2			; Display2 off
	MOVF	CENTENA,W		; copia o dígito em W
	CALL	BIN8DEC			;
	MOVF	D0,W			;
	CALL	DECOD_DISPLAY	; decodifica o dígito
	CALL	DECOD_SEGMENTS
	BCF		DISP3			; ativa display 3
	CLRW					; W=0
	GOTO	MAINCONTINUE

TESTLEDS:
	BTFSS	LEDENINT		; Show Leds?
	GOTO	MAINCONTINUE
	BSF		DISP3			; Desativa display 3
	BCF		LED_UP			; Liga led UP
	BCF		LED_DWN			; Liga led DWN
	BCF		LED_TEMP		; Liga led TEMP
	BSF		LED_TIMER		; Desliga led TIMER
	BSF		LED_LIGHT		; Desliga led LIGHT
	BSF		LED_TOASTER		; Desliga led TOASTER
	BCF		LED_CMN			; Liga Leds

MAINCONTINUE:
	CALL	CHECKKEYS		; verifica teclas
	GOTO	MAIN_LOOP		; retorna para o LOOP_PRINCIPAL

	END
